import React from 'react'
import { omit } from 'lodash'
import styled, { withTheme } from 'styled-components'
import { Button, Icon, TextField } from '@pubsweet/ui'
import { states } from 'config'
import { Portal, Buttons, Close, CloseButton } from '../ui'
import { ManuscriptMutations, NoteMutations } from '../SubmissionMutations'
import ResolveDuplicates from '../ResolveDuplicates'
import GrantSearch from '../GrantSearch'
import CitationEdit, { Exit } from './CitationEdit'

const RouteButton = styled(Button)`
  align-items: center;
  display: inline-flex;
`
class EmbargoEdit extends React.Component {
  state = { embargo: this.props.embargo }
  render() {
    return (
      <React.Fragment>
        <TextField
          label="Enter embargo in months"
          name="embargo"
          onChange={e => this.setState({ embargo: e.target.value })}
          value={this.state.embargo}
        />
        <Buttons right>
          <Button
            onClick={() => {
              this.props.change(this.state.embargo)
              this.props.close()
            }}
            primary
          >
            Save
          </Button>
          <Exit close={this.props.close} />
        </Buttons>
      </React.Fragment>
    )
  }
}

const DuplicatesWithMutations = NoteMutations(ResolveDuplicates)

const MetaEdit = withTheme(
  ({ theme, close, manuscript, duplicates, toEdit, ...props }) => {
    const { meta } = manuscript
    const { fundingGroup: grants, releaseDelay, notes } = meta
    const fundingGroup = grants
      ? grants.map(g => {
          const n = omit(g, '__typename')
          n.pi = omit(g.pi, '__typename')
          return n
        })
      : []
    const qa = states.indexOf('submitted')
    const xml = states.indexOf('xml-triage')
    const done = states.indexOf('ncbi-ready')
    const curr = states.indexOf(manuscript.status)
    if (toEdit === 'dupes' && duplicates) {
      return (
        <DuplicatesWithMutations
          close={close}
          duplicates={duplicates}
          manuscript={manuscript}
          note={notes ? notes.find(n => n.notesType === 'notDuplicates') : null}
        />
      )
    }
    return (
      <Portal style={{ backgroundColor: theme.colorBackground }} transparent>
        <Close>
          <CloseButton onClick={close} />
        </Close>
        <React.Fragment>
          {(() => {
            switch (toEdit) {
              case 'grants':
                return (
                  <React.Fragment>
                    <GrantSearch
                      changedGrants={props.updateGrants}
                      hideEmbargo
                      selectedGrants={fundingGroup}
                    />
                    <Exit close={close} />
                  </React.Fragment>
                )
              case 'embargo':
                return (
                  <EmbargoEdit
                    change={props.updateEmbargo}
                    close={close}
                    embargo={releaseDelay}
                  />
                )
              case 'citation':
                return (
                  <CitationEdit
                    change={props.changeCitation}
                    close={close}
                    manuscript={manuscript}
                  />
                )
              case 'status':
                return (
                  <React.Fragment>
                    {curr > qa && curr !== xml ? (
                      <p>
                        <RouteButton
                          onClick={() =>
                            props.setStatus(
                              curr > xml ? ' xml-triage' : ' submitted',
                              close,
                            )
                          }
                        >
                          <Icon color="currentColor" size={2.5}>
                            send
                          </Icon>
                          Send back
                        </RouteButton>
                        {` to most recent editing status:`}
                        {curr > xml ? ' xml-triage' : ' submitted'}
                      </p>
                    ) : (
                      <p>
                        {`The submission is currently in an editing state (`}
                        {manuscript.status}
                        {`). You may route it elsewhere as usual from the manuscript view.`}
                      </p>
                    )}
                    <p>
                      {curr >= done ? (
                        <RouteButton>
                          <Icon color="currentColor" size={2.5}>
                            delete
                          </Icon>
                          Withdraw manuscript
                        </RouteButton>
                      ) : (
                        <RouteButton>
                          <Icon color="currentColor" size={2.5}>
                            trash-2
                          </Icon>
                          Remove manuscript
                        </RouteButton>
                      )}
                    </p>
                    <Exit close={close} />
                  </React.Fragment>
                )
              default:
                return null
            }
          })()}
        </React.Fragment>
      </Portal>
    )
  },
)

export default ManuscriptMutations(MetaEdit)
